@@ -2,11 +2,10 @@ package main
 
 import (
 	"bufio"
-	"crypto/rand"
+	"math/rand"
 	"encoding/json"
 	"fmt"
 	"io"
-	"math/big"
 	"os"
 	"os/exec"
 	"regexp"
@@ -25,13 +24,12 @@ import (
 )
 
 const (
-	version		float32	=	12.99
+	version		float32	=	13
 	controlFile	string	=	"instanceId=inIdHold\nappID=idHold\nbusDir=busHold\nbusDirAy=busAyHold\nfriendlyName=friendlyHold"
 )
 
 type RUNTIME_OPT struct {
 	action		bool
-	fullCmdline	string
 	argStop		bool
 	applicationArgs	[]string
 	userExpose	string
@@ -43,7 +41,6 @@ type RUNTIME_PARAMS struct {
 	instanceID		string
 	waylandDisplay		string
 	bwCmd			[]string
-	sdEnvParm		[]string
 }
 
 type XDG_DIRS struct {
@@ -139,7 +136,6 @@ var (
 	envsChan		= make(chan string, 512)
 	envsFlushReady		= make(chan int8, 1)
 	startAct		string
-	launchTarget		= make(chan string, 1)
 	signalWatcherReady	= make(chan int8, 1)
 	gpuChan 		= make(chan []string, 1)
 	busArgChan		= make(chan []string, 1)
@@ -153,27 +149,54 @@ var (
 					"/sys/module/nvidia_uvm",
 					"/sys/module/nvidia_wmi_ec_backlight",
 				}
+	pechoChan		= make(chan []string, 128)
 )
 
-func pecho(level string, message string) {
-	switch level {
+func pechoWorker() {
+	var externalLoggingLevel = os.Getenv("PORTABLE_LOGGING")
+	switch externalLoggingLevel {
 		case "debug":
-			if internalLoggingLevel <= 1 {
-				fmt.Println("[Debug] ", message)
-			}
+			internalLoggingLevel = 1
 		case "info":
-			if internalLoggingLevel <= 2 {
-				fmt.Println("[Info] ", message)
-			}
+			internalLoggingLevel = 2
 		case "warn":
-			fmt.Println("[Warn] ", message)
-		case "crit":
-			fmt.Println("[Critical] ", message)
-			stopApp()
-			panic("A critical error has happened")
+			internalLoggingLevel = 3
 		default:
-			fmt.Println("[Undefined] ", message)
+			internalLoggingLevel = 3
 	}
+	pechoChan <- []string{
+		"debug",
+		"Initialized logging daemon",
+	}
+	for {
+		chanRes := <- pechoChan
+		switch chanRes[0] {
+			case "debug":
+				if internalLoggingLevel <= 1 {
+					fmt.Println("[Debug] ", chanRes[1])
+				}
+			case "info":
+				if internalLoggingLevel <= 2 {
+					fmt.Println("[Info] ", chanRes[1])
+				}
+			case "warn":
+				fmt.Println("[Warn] ", chanRes[1])
+			case "crit":
+				fmt.Println("[Critical] ", chanRes[1])
+				stopApp()
+				panic("A critical error has happened")
+			default:
+				fmt.Println("[Undefined] ", chanRes[1])
+		}
+	}
+}
+
+func pecho(level string, message string) {
+	var msgSlice = []string{
+		level,
+		message,
+	}
+	pechoChan <- msgSlice
 }
 
 
@@ -270,7 +293,6 @@ func showStats() {
 }
 
 func cmdlineDispatcher(cmdChan chan int8) {
-	runtimeOpt.fullCmdline = strings.Join(os.Args, ", ")
 	cmdlineArray := os.Args
 	for index, value := range cmdlineArray {
 		if index == 0 {
@@ -289,6 +311,10 @@ func cmdlineDispatcher(cmdChan chan int8) {
 		switch value {
 			case "--actions" :
 			runtimeOpt.action = true
+			if len(cmdlineArray) <= index + 1 {
+				pecho("warn", "--actions require an argument")
+				break
+			}
 			switch cmdlineArray[index + 1] {
 				case "quit":
 					runtimeOpt.miTerminate = true
@@ -344,9 +370,9 @@ func cmdlineDispatcher(cmdChan chan int8) {
 	}
 	addEnv("targetArgs=" + string(encodedArg))
 	cmdChan <- 1
-	pecho("debug", "Full command line: " + runtimeOpt.fullCmdline)
+	fullCmdline := strings.Join(cmdlineArray, ", ")
+	pecho("debug", "Full command line: " + fullCmdline)
 	pecho("info", "Application arguments: " + strings.Join(runtimeOpt.applicationArgs, ", "))
-
 }
 
 func shareFile() {
@@ -385,17 +411,6 @@ func shareFile() {
 }
 
 func getVariables() {
-	var externalLoggingLevel = os.Getenv("PORTABLE_LOGGING")
-	switch externalLoggingLevel {
-		case "debug":
-			internalLoggingLevel = 1
-		case "info":
-			internalLoggingLevel = 2
-		case "warn":
-			internalLoggingLevel = 3
-		default:
-			internalLoggingLevel = 3
-	}
 	runtimeOpt.userExpose = os.Getenv("bwBindPar")
 	runtimeOpt.userLang = os.Getenv("LANG")
 }
@@ -555,11 +570,11 @@ func readConf() {
 func mkdirPool(dirs []string) {
 	var wg sync.WaitGroup
 	for _, dir := range dirs {
-		wg.Add(1)
-		go func () {
-			defer wg.Done()
-			os.MkdirAll(dir, 0700)
-		} ()
+		wg.Go(
+			func() {
+				os.MkdirAll(dir, 0700)
+			},
+		)
 	}
 	wg.Wait()
 }
@@ -568,13 +583,12 @@ func genInstanceID(genInfo chan int8, proceed chan int8) {
 	var wg sync.WaitGroup
 	pecho("debug", "Generating instance ID")
 	for {
-		genId, _ := rand.Int(rand.Reader, big.NewInt(9999999999))
-		idCandidate := int(genId.Int64())
+		idCandidate := rand.Intn(2147483647)
 		pecho("debug", "Trying instance ID: " + strconv.Itoa(idCandidate))
 		_, err := os.Stat(xdgDir.runtimeDir + "/.flatpak/" + strconv.Itoa(idCandidate))
 		if err == nil {
 			pecho("warn", "Unable to use instance ID " + strconv.Itoa(idCandidate))
-		} else if genId.Int64() < 1024 {
+		} else if idCandidate < 1024 {
 			pecho("debug", "Rejecting low ID")
 		} else {
 			runtimeInfo.instanceID = strconv.Itoa(idCandidate)
@@ -615,26 +629,17 @@ func genInstanceID(genInfo chan int8, proceed chan int8) {
 func writeControlFile() {
 	os.MkdirAll(xdgDir.runtimeDir + "/portable/" + confOpts.appID, 0700)
 	var controlContent = controlFile
-	controlContent = strings.ReplaceAll(controlContent, "inIdHold", runtimeInfo.instanceID)
-	controlContent = strings.ReplaceAll(controlContent, "idHold", confOpts.appID)
-	controlContent = strings.ReplaceAll(
-		controlContent,
-		"busHold",
-		xdgDir.runtimeDir + "/app/" + confOpts.appID,
-	)
-	controlContent = strings.ReplaceAll(
-		controlContent,
-		"busAyHold",
-		xdgDir.runtimeDir + "/app/" + confOpts.appID + "-a11y",
-	)
-	controlContent = strings.ReplaceAll(
-		controlContent,
-		"friendlyHold",
-		confOpts.friendlyName,
+	replacer := strings.NewReplacer(
+		"inIdHold",	runtimeInfo.instanceID,
+		"idHold",	confOpts.appID,
+		"busHold",	xdgDir.runtimeDir + "/app/" + confOpts.appID,
+		"busAyHold",	xdgDir.runtimeDir + "/app/" + confOpts.appID + "-a11y",
+		"friendlyHold",	confOpts.friendlyName,
 	)
+
 	os.WriteFile(
 		xdgDir.runtimeDir + "/portable/" + confOpts.appID + "/control",
-		[]byte(controlContent),
+		[]byte(replacer.Replace(controlContent)),
 		0700,
 	)
 }
@@ -1118,16 +1123,18 @@ func startProxy(conn *dbus.Conn, ctx context.Context) {
 		defer wg.Done()
 		dbusArgs = <- busArgChan
 	} ()
+	wg.Go(func() {
+		os.MkdirAll(
+			xdgDir.runtimeDir + "/.flatpak/" + runtimeInfo.instanceID,
+			0700,
+		)
+	})
 	go func () {
 		defer wg.Done()
 		os.MkdirAll(
 			xdgDir.runtimeDir + "/app/" + confOpts.appID,
 			0700,
 		)
-		os.MkdirAll(
-			xdgDir.runtimeDir + "/.flatpak/" + runtimeInfo.instanceID,
-			0700,
-		)
 	} ()
 	var unitWants = []string{
 		"xdg-document-portal.service",
@@ -1330,7 +1337,7 @@ func instDesktopFile() {
 	}
 }
 
-func setXDGEnvs (xdgEnvReady chan int8) {
+func setXDGEnvs() {
 	addEnv("XDG_CONFIG_HOME=" + translatePath(xdgDir.confDir))
 	addEnv("XDG_DOCUMENTS_DIR=" + xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Documents")
 	addEnv("XDG_DATA_HOME=" + xdgDir.dataDir + "/" + confOpts.stateDirectory + "/.local/share")
@@ -1343,10 +1350,9 @@ func setXDGEnvs (xdgEnvReady chan int8) {
 	addEnv("XDG_MUSIC_DIR=" + xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Music")
 	addEnv("XDG_PICTURES_DIR=" + xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Pictures")
 	addEnv("XDG_VIDEOS_DIR=" + xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Videos")
-	xdgEnvReady <- 1
 }
 
-func imEnvs (imReady chan int8) {
+func imEnvs () {
 	addEnv("IBUS_USE_PORTAL=1")
 	var imKind string
 	if confOpts.waylandOnly == true {
@@ -1420,18 +1426,16 @@ func imEnvs (imReady chan int8) {
 				}
 		}
 	}
-	imReady <- 1
 }
 
-func setupSharedDir (shareDir chan int8) {
+func setupSharedDir () {
 	os.MkdirAll(xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Shared", 0700)
 	os.Link(
 		xdgDir.dataDir + "/" + confOpts.stateDirectory + "/Shared",
 		xdgDir.dataDir + "/" + confOpts.stateDirectory + "/共享文件")
-	shareDir <- 1
 }
 
-func miscEnvs (mEnvRd chan int8) {
+func miscEnvs () {
 	if confOpts.useZink == true {
 		addEnv("__GLX_VENDOR_LIBRARY_NAME=mesa")
 		addEnv("MESA_LOADER_DRIVER_OVERRIDE=zink")
@@ -1461,59 +1465,80 @@ func miscEnvs (mEnvRd chan int8) {
 	addEnv("XDG_SESSION_TYPE=" + os.Getenv("XDG_SESSION_TYPE"))
 	addEnv("WAYLAND_DISPLAY=" + xdgDir.runtimeDir + "/wayland-0")
 	addEnv("DBUS_SESSION_BUS_ADDRESS=unix:path=/run/sessionBus")
-	mEnvRd <- 1
 }
 
 func prepareEnvs() {
-	imChan := make(chan int8, 1)
-	xdgEnvChan := make(chan int8, 1)
-	shareDirChan := make(chan int8, 1)
-	miscEnvChan := make(chan int8, 1)
-	go imEnvs(imChan)
-	go setXDGEnvs(xdgEnvChan)
-	go setupSharedDir(shareDirChan)
-	go miscEnvs(miscEnvChan)
-	userEnvs, err := os.OpenFile(xdgDir.dataDir + "/" + confOpts.stateDirectory + "/portable.env", os.O_RDONLY, 0700)
-	if err != nil {
-		pecho("info", "Unable to read user defined environment variables: " + err.Error())
-		if os.IsNotExist(err) {
-			var template string = "# This file accepts simple KEY=VAL envs"
-			os.WriteFile(
-				xdgDir.dataDir + "/" + confOpts.stateDirectory + "/portable.env",
-				[]byte(template),
-				0700,
-			)
+	var wg sync.WaitGroup
+	wg.Go(func() {
+		imEnvs()
+	})
+	wg.Go(func() {
+		setXDGEnvs()
+	})
+	wg.Go(func() {
+		miscEnvs()
+	})
+	wg.Go(func() {
+		userEnvs, err := os.OpenFile(
+			xdgDir.dataDir + "/" + confOpts.stateDirectory + "/portable.env",
+			os.O_RDONLY,
+			0700,
+		)
+		if err != nil {
+			if os.IsNotExist(err) {
+				const template = "# This file accepts simple KEY=VAL envs"
+				os.WriteFile(
+					xdgDir.dataDir + "/" + confOpts.stateDirectory + "/portable.env",
+					[]byte(template),
+					0700,
+				)
+			} else {
+				pecho(
+				"warn",
+				"Unable to open file for reading environment variables: " + err.Error(),
+				)
+			}
 		} else {
-			pecho("warn", "Unable to open file for reading environment variables: " + err.Error())
-		}
-	} else {
-		userEnvRead, errRead := io.ReadAll(userEnvs)
-		if errRead != nil {
-			pecho("warn", "I/O error reading environment variables: " + errRead.Error())
-		} else {
-			lines := strings.Split(strings.TrimRight(string(userEnvRead), "\n"), "\n")
-			for _, line := range lines {
+			defer userEnvs.Close()
+			scanner := bufio.NewScanner(userEnvs)
+			for scanner.Scan() {
+				if scanner.Err() != nil {
+					pecho(
+					"warn",
+					"Could not read user environment variables" + scanner.Err().Error(),
+					)
+					continue
+				}
+				line := scanner.Text()
 				addEnv(line)
 			}
 		}
-	}
-	packageEnvs, errPkg := os.OpenFile(confOpts.confPath, os.O_RDONLY, 0700)
-	if errPkg != nil {
-		pecho("crit", "Could not open package config " + confOpts.confPath + ": " + errPkg.Error())
-	}
-	pkgRead, errPkgR := io.ReadAll(packageEnvs)
-	if errPkgR != nil {
-		pecho("crit", "I/O error reading config: " + errPkgR.Error())
-	}
-	pkgEnv := strings.Split(strings.TrimRight(string(pkgRead), "\n"), "\n")
-	for _, line := range pkgEnv {
-		addEnv(line)
-	}
+	})
 
-	<- shareDirChan
-	<- miscEnvChan
-	<- xdgEnvChan
-	<- imChan
+	wg.Go(func() {
+		packageEnvs, errPkg := os.OpenFile(confOpts.confPath, os.O_RDONLY, 0700)
+		if errPkg != nil {
+			pecho(
+				"crit",
+				"Could not open package config " + confOpts.confPath + ": " + errPkg.Error(),
+				)
+		} else {
+			defer packageEnvs.Close()
+			scanner := bufio.NewScanner(packageEnvs)
+			for scanner.Scan() {
+				if scanner.Err() != nil {
+					pecho(
+						"crit",
+						"Could not read package defined environment variables",
+						)
+				}
+				line := scanner.Text()
+				addEnv(line)
+			}
+		}
+	})
+
+	wg.Wait()
 }
 
 func genBwArg(
@@ -1521,9 +1546,9 @@ func genBwArg(
 	xChan chan []string,
 	camChan chan []string,
 	inputChan chan []string,
+	wayDisplayChan chan []string,
 	) {
-	wayDisplayChan := make(chan[]string, 1)
-	go waylandDisplay(wayDisplayChan)
+
 	miscChan := make(chan []string, 1)
 	go miscBinds(miscChan, pwChan)
 
@@ -1770,20 +1795,22 @@ func genBwArg(
 		wayArgs...
 	)
 
-	miscArgs := <- miscChan
-	runtimeInfo.bwCmd = append(
-		runtimeInfo.bwCmd,
-		miscArgs...
-	)
-
-	if confOpts.bindInputDevices == true {
-		inputArgs := <- inputChan
+	for arg := range miscChan {
 		runtimeInfo.bwCmd = append(
 			runtimeInfo.bwCmd,
-			inputArgs...
+			arg...
 		)
 	}
 
+	if confOpts.bindInputDevices == true {
+		for inputArg := range inputChan {
+			runtimeInfo.bwCmd = append(
+				runtimeInfo.bwCmd,
+				inputArg...
+			)
+		}
+	}
+
 
 	camArgs := <- camChan
 	runtimeInfo.bwCmd = append(
@@ -1807,10 +1834,11 @@ func genBwArg(
 
 func flushEnvs() {
 	//os.MkdirAll(xdgDir.runtimeDir + "/portable/" + confOpts.appID, 0700)
+	var envs = []string{}
 
 	for env := range envsChan {
-		runtimeInfo.sdEnvParm = append(
-			runtimeInfo.sdEnvParm,
+		envs = append(
+			envs,
 			env,
 		)
 	}
@@ -1828,14 +1856,15 @@ func flushEnvs() {
 	}
 	defer fd.Close()
 	writer := bufio.NewWriter(fd)
-	for _, env := range runtimeInfo.sdEnvParm {
-		_, err = fmt.Fprintln(writer, env)
+	for _, env := range envs {
+		writer.WriteString(env)
+		writer.WriteString("\n")
 	}
 	err = writer.Flush()
 	if err != nil {
 		pecho("crit", "Could not write environment variables: " + err.Error())
 	}
-	envsFlushReady <- 1
+	close(envsFlushReady)
 }
 
 func translatePath(input string) (output string) {
@@ -1858,8 +1887,88 @@ func maskDir(path string) (maskArgs []string) {
 }
 
 func miscBinds(miscChan chan []string, pwChan chan []string) {
+	var wg sync.WaitGroup
 	var miscArgs = []string{}
 
+	wg.Go(func() {
+		miscChan <- maskDir("/proc/bus")
+	})
+
+	wg.Go(func() {
+		miscChan <- maskDir("/proc/driver")
+	})
+
+	wg.Go(func() {
+		_, err := os.Stat("/usr/lib/flatpak-xdg-utils/flatpak-spawn")
+		if err == nil {
+			miscChan <- []string{
+				"--ro-bind",
+				"/usr/lib/portable/overlay-usr/flatpak-spawn",
+				"/usr/lib/flatpak-xdg-utils/flatpak-spawn",
+			}
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- maskDir("/etc/kernel")
+	})
+
+	wg.Go(func() {
+		miscChan <- []string{
+			"--ro-bind-try",
+			xdgDir.confDir + "/fontconfig",
+			translatePath(xdgDir.confDir + "/fontconfig"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string {
+			"--ro-bind-try",
+			xdgDir.confDir + "/gtk-3.0/gtk.css",
+			translatePath(xdgDir.confDir + "/gtk-3.0/gtk.css"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string {
+			"--ro-bind-try",
+			xdgDir.confDir + "/gtk-3.0/colors.css",
+			translatePath(xdgDir.confDir + "/gtk-3.0/colors.css"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string {
+			"--ro-bind-try",
+			xdgDir.confDir + "/gtk-4.0/gtk.css",
+			translatePath(xdgDir.confDir + "/gtk-4.0/gtk.css"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string{
+			"--ro-bind-try",
+			xdgDir.confDir + "/qt6ct",
+			translatePath(xdgDir.confDir + "/qt6ct"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string{
+			"--ro-bind-try",
+			xdgDir.dataDir + "/fonts",
+			translatePath(xdgDir.dataDir + "/fonts"),
+		}
+	})
+
+	wg.Go(func() {
+		miscChan <- []string{
+			"--ro-bind-try",
+			xdgDir.dataDir + "/icons",
+			translatePath(xdgDir.dataDir + "/icons"),
+		}
+	})
+
 	if len(runtimeOpt.userExpose) > 0 {
 		_, err := os.Stat(runtimeOpt.userExpose)
 		if err != nil {
@@ -1906,15 +2015,6 @@ func miscBinds(miscChan chan []string, pwChan chan []string) {
 			}
 		}
 	}
-
-	miscArgs = append(
-		miscArgs,
-		maskDir("/proc/bus")...
-	)
-	miscArgs = append(
-		miscArgs,
-		maskDir("/proc/driver")...
-	)
 	pwArgs := <- pwChan
 	miscArgs = append(
 		miscArgs,
@@ -1951,50 +2051,12 @@ func miscBinds(miscChan chan []string, pwChan chan []string) {
 		)
 	}
 
-	_, err := os.Stat("/usr/lib/flatpak-xdg-utils/flatpak-spawn")
-	if err == nil {
-		miscArgs = append(
-			miscArgs,
-			"--ro-bind",
-			"/usr/lib/portable/overlay-usr/flatpak-spawn",
-			"/usr/lib/flatpak-xdg-utils/flatpak-spawn",
-		)
-	}
 
-	dirFd, errRead := os.Stat("/etc/kernel")
-	if errRead == nil && dirFd.IsDir() == true {
-		miscArgs = append(
-			miscArgs,
-			"--tmpfs",
-			"/etc/kernel",
-		)
-	}
-	miscArgs = append(
-		miscArgs,
-		"--ro-bind-try",
-			xdgDir.confDir + "/fontconfig",
-			translatePath(xdgDir.confDir + "/fontconfig"),
-		"--ro-bind-try",
-			xdgDir.confDir + "/gtk-3.0/gtk.css",
-			translatePath(xdgDir.confDir + "/gtk-3.0/gtk.css"),
-		"--ro-bind-try",
-			xdgDir.confDir + "/gtk-3.0/colors.css",
-			translatePath(xdgDir.confDir + "/gtk-3.0/colors.css"),
-		"--ro-bind-try",
-			xdgDir.confDir + "/gtk-4.0/gtk.css",
-			translatePath(xdgDir.confDir + "/gtk-4.0/gtk.css"),
-		"--ro-bind-try",
-			xdgDir.confDir + "/qt6ct",
-			translatePath(xdgDir.confDir + "/qt6ct"),
-		"--ro-bind-try",
-			xdgDir.dataDir + "/fonts",
-			translatePath(xdgDir.dataDir + "/fonts"),
-		"--ro-bind-try",
-			xdgDir.dataDir + "/icons",
-			translatePath(xdgDir.dataDir + "/icons"),
-	)
+
 
 	miscChan <- miscArgs
+	wg.Wait()
+	close(miscChan)
 }
 
 func bindXAuth(xauthChan chan []string) {
@@ -2157,7 +2219,9 @@ func gpuBind(gpuChan chan []string) {
 						bindCard(card, arg)
 					} (cardName, argChan)
 				}
-				argChan <- tryBindNv()
+				wg.Go(
+					func() {argChan <- tryBindNv()},
+				)
 			} else {
 				for idx, cardName := range totalGpus {
 					wg.Add(1)
@@ -2479,13 +2543,19 @@ func inputBind(inputBindChan chan []string) {
 	}
 
 	inputBindChan <- inputBindArg
+	close(inputBindChan)
 	pecho("debug", "Finished calculating input arguments: " + strings.Join(inputBindArg, " "))
 }
 
 func multiInstance(miChan chan bool) {
-	pecho("debug", "Dialing daemon socket...")
 	var socketPath string = xdgDir.runtimeDir + "/portable/"
 	socketPath = socketPath + confOpts.appID + "/portable-control/daemon"
+	_, errStat := os.Stat(socketPath)
+	if errStat != nil && os.IsNotExist(errStat) {
+		miChan <- false
+		return
+	}
+	pecho("debug", "Dialing daemon socket...")
 	_, err := net.Dial("unix", socketPath)
 	socketPath = xdgDir.runtimeDir + "/portable/"
 	socketPath = socketPath + confOpts.appID + "/portable-control/auxStart"
@@ -2496,7 +2566,6 @@ func multiInstance(miChan chan bool) {
 			os.Exit(2)
 		}
 		miChan <- false
-		pecho("debug", "Starting new instance...")
 		return
 	} else {
 		pecho("debug", "Another instance running")
@@ -2629,7 +2698,8 @@ func waitChan(tgChan chan int8, chanName string) {
 }
 
 func main() {
-	//runtime.SetBlockProfileRate(1)
+	go pechoWorker()
+
 	var wg sync.WaitGroup
 	go getVariables()
 	wg.Add(1)
@@ -2650,12 +2720,15 @@ func main() {
 	} ()
 	go stopAppWorker(conn, sdCancelFunc, sdContext)
 
-	inputChan := make(chan []string, 1)
+	inputChan := make(chan []string, 512)
 	go inputBind(inputChan) // This is fine, since genBwArg takes care of conf switching
 	fmt.Println("Portable daemon", version, "starting")
 	cmdChan := make(chan int8, 1)
 	wg.Wait()
 
+	wayDisplayChan := make(chan[]string, 1)
+	go waylandDisplay(wayDisplayChan)
+
 	go cmdlineDispatcher(cmdChan)
 	go gpuBind(gpuChan)
 	wg.Add(1)
@@ -2663,6 +2736,9 @@ func main() {
 		defer wg.Done()
 		instDesktopFile()
 	} ()
+	wg.Go(func() {
+		setupSharedDir()
+	})
 	genChan := make(chan int8, 2) /* Signals when an ID has been chosen,
 		and we signal back when multi-instance is cleared
 		in another channel */
@@ -2712,7 +2788,7 @@ func main() {
 	<- genChan // Stage one, ensures that IDs are actually present
 	go func () {
 		defer wg.Done()
-		genBwArg(pwSecContextChan, xChan, camChan, inputChan)
+		genBwArg(pwSecContextChan, xChan, camChan, inputChan, wayDisplayChan)
 	} ()
 	genChanProceed <- 1
 	go calcDbusArg(busArgChan)
